from django.http import Http404

class method_restricted_to(object):
    def __init__(self, methods=[]):
        self.methods = methods

    def __call__(self, f):
        def wrapped_f(request, *args):
            if request.method in self.methods:
                return f(request, *args)
            else:
                raise Http404
        return wrapped_f

class is_ajax(object):
    def __call__(self, f):
        def wrapped_f(request, *args):
            if request.is_ajax():
                return f(request, *args)
            else:
                raise Http404
        return wrapped_f


__author__ = 'juliensyx'
from django.conf import settings
from django.template.loader import BaseLoader
from django.template.base import Template, TemplateDoesNotExist

class Loader(BaseLoader):
    is_usable = True

    def load_template(self, template_name, template_dirs=None):
        source, origin = self.load_template_source(template_name, template_dirs)
        template = Template(source)
        return template, origin

    def load_template_source(self, template_name, template_dirs=None):
        try:
            with open(template_name, 'rb') as fp:
                return (fp.read().decode(settings.FILE_CHARSET), template_name)
        except IOError:
            raise TemplateDoesNotExist("Template does not exist")



from datetime import datetime
from twisted.python import log
from pymongo import MongoClient
from tasks import ScheduledTask
from dateutil.rrule import *
from twisted.python.reflect import namedAny
from twisted.python import log
from sys import path

class TraceTask(ScheduledTask):

    def before_execute(self):
        log.msg('current time is: %s' % datetime.now())
        log.msg('tasks last runtime was: %s' % self.last_runtime)
        log.msg('tasks next runtime is: %s' % self.next_scheduled_runtime)

    def after_execute(self):
        log.msg('current time is: %s' % datetime.now())
        log.msg('tasks last runtime was: %s' % self.last_runtime)
        log.msg('tasks next runtime is: %s' % self.next_scheduled_runtime)

class ScheduledTaskManager(object):
    '''Manages a group of tasks.
    '''
    def __init__(self, configuration):
        self.configuration = configuration
        mongo = MongoClient(configuration['database']['server'], configuration['database']['port'])
        self.database = mongo.lisa
        self.build_tasks()

    def build_tasks(self):
        self.tasks = []
        for cron in self.database.crons.find( { "enabled": True } ):
            rule = rrulestr(str(cron['rule']))
            object = namedAny(cron['module'] + '.' + cron['class'])()
            func = namedAny(cron['module'] + '.' + cron['class'] + '.' + cron['method'])
            if cron['args']:
                if cron['debug'] == True:
                    self.tasks.append(TraceTask(cron['name'], rule, func, object, cron['args']))
                else:
                    self.tasks.append(ScheduledTask(cron['name'], rule, func, object, cron['args']))
            else:
                if cron['debug'] == True:
                    self.tasks.append(TraceTask(cron['name'], rule, func, object))
                else:
                    self.tasks.append(ScheduledTask(cron['name'], rule, func, object))

    def add_task(self, task):
        '''Adds a task to be run.
        
        Expects a :class:`txscheduler.tasks.ScheduledTask` instance.
        '''
        self.tasks.append(task)

    def reload(self):
        '''Reload the self.tasks
        '''
        self.build_tasks()
        return "OK"

    def remove_task(self, task):
        '''Removes a task to be run.
        
        Expects a :class:`txscheduler.tasks.ScheduledTask` instance.
        '''
        self.tasks.remove(task)
        
    def run(self):
        '''Checks for tasks which need to be run and runs them.
        '''
        if self.configuration['debug']['debug_scheduler'] == True:
            log.msg('ScheduledTaskManager: checking for tasks to run...')
            log.msg(self.tasks)
        tasks_to_run = [task for task in self.tasks if
                            task.next_scheduled_runtime < datetime.now()]
        if self.configuration['debug']['debug_scheduler'] == True:
            log.msg('Scheduledtaskmanager: %d tasks found.' % len(tasks_to_run))
        for task in tasks_to_run:
            task.run()



# -*- coding: UTF-8 -*-
import json, os, gettext
from pymongo import MongoClient
from twisted.python.reflect import namedAny
from twisted.python import log
from wit import Wit
import lisa.server


class RulesEngine():
    def __init__(self, configuration):
        self.configuration = configuration
        client = MongoClient(self.configuration['database']['server'], self.configuration['database']['port'])
        self.database = client.lisa

        path = os.path.normpath(str(lisa.server.__path__[0]) + "/lang")
        _ = translation = gettext.translation(domain='intents', localedir=path, fallback=True,
                                              languages=[self.configuration['lang']]).ugettext
        self.wit = Wit(self.configuration['wit_token'])

    def Rules(self, jsonData, lisaprotocol):
        rulescollection = self.database.rules
        intentscollection = self.database.intents
        jsonInput = self.wit.get_message(unicode(jsonData['body']))
        jsonInput['from'], jsonInput['type'], jsonInput['zone'] = jsonData['from'], jsonData['type'], jsonData['zone']

        if self.configuration['debug']['debug_before_before_rule']:
            log.msg("Before 'before' rule: " + str(jsonInput))
        for rule in rulescollection.find({"enabled": True, "before": {"$ne":None}}).sort([("order", 1)]):
            exec(rule['before'])
        if self.configuration['debug']['debug_after_before_rule']:
            log.msg("After 'before' rule: " + str(jsonInput))
        if self.configuration['debug']['debug_wit']:
            log.msg("WIT: " + str(jsonInput['outcome']))

        oIntent = intentscollection.find_one({"name": jsonInput['outcome']['intent']})
        if oIntent and jsonInput['outcome']['confidence'] >= self.configuration['wit_confidence']:
            instance = namedAny(str(oIntent["module"]))()
            methodToCall = getattr(instance, oIntent['function'])
            jsonOutput = methodToCall(jsonInput)
        else:
            jsonOutput = {}
            jsonOutput['plugin'] = "None"
            jsonOutput['method'] = "None"
            jsonOutput['body'] = self._("I have not the right plugin installed to answer you correctly")
        jsonOutput['from'] = jsonData['from']
        if self.configuration['debug']['debug_before_after_rule']:
            log.msg("Before 'after' rule: " + str(jsonOutput))
        for rule in rulescollection.find({"enabled": True, "after": {"$ne":None}}).sort([("order", 1)]):
            exec(rule['after'])
            #todo it doesn't check if the condition of the rule after has matched to end the rules
            if rule['end']:
                break
        if self.configuration['debug']['debug_after_after_rule']:
            log.msg("After 'after' rule: " + str(jsonOutput))



from django.http import HttpResponse
from django.shortcuts import render
from django.contrib.auth.decorators import login_required
from lisa.server.service import configuration

@login_required()
def index(request):
    if configuration['enable_secure_mode']:
        websocket = 'wss'
    else:
        websocket = 'ws'
    context = {
        'websocket': websocket,
        'lang': configuration['lang']
    }
    return render(request, 'googlespeech/index.html', context)



from lisa.server.libs.server import LisaFactorySingleton
from twisted.trial import unittest
from twisted.test import proto_helpers

import json

class LisaClientTestCase(unittest.TestCase):
    def setUp(self):
        factory = LisaFactorySingleton.get()
        self.proto = factory.buildProtocol(('127.0.0.1', 0))
        self.tr = proto_helpers.StringTransport()
        self.proto.makeConnection(self.tr)

    def convertjson(self, type=None, message=None, command=None, argdict=None, zone="zone1"):
        jsondict = {"from": "TestClient"}
        if message:
            jsondict['body'] = unicode(message)
        if type:
            jsondict['type'] = type
        if command:
            jsondict['command'] = command
        if command:
            jsondict['command'] = command
        if zone:
            jsondict['zone'] = zone
        if argdict:
            for k,v in argdict.items():
                jsondict[k] = v

        return json.dumps(jsondict)

    #def test_receivechat(self):
    #    json = self.convertjson(message="")

    def test_receive_command_login(self):
        message = "LOGIN"
        argdict = {}
        datajson = self.convertjson(message=message, type="command", command="LOGIN", argdict=argdict, zone="zone1")
        self.proto.lineReceived(data=datajson)
        answer = json.loads(self.tr.value())
        self.assertEqual(answer['bot_name'], "lisa")



# -*- coding: utf-8; tab-width: 4; indent-tabs-mode: t; python-indent: 4 -*-
from pymongo import MongoClient
from lisa.server.service import configuration
from lisa.server.libs import LisaFactorySingleton


class IPlugin(object):
    """
    The most simple interface to be inherited when creating a plugin.
    """

    def __init__(self):
        """
        Set the basic variables.
        """
        self.lisa = LisaFactorySingleton.get()
        self.configuration_lisa = configuration
        self.mongo = MongoClient(host=self.configuration_lisa['database']['server'],
                            port=self.configuration_lisa['database']['port'])


from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template import Context, loader
from django.http import HttpResponse

from django.contrib.auth.decorators import login_required
import os,json

from lisa.server.web.weblisa.utils import method_restricted_to, is_ajax
from lisa.server.web.weblisa.settings import LISA_PATH

from lisa.plugins.TestPlugin.modules.testplugin import TestPlugin

# Template system will be fixed with Django 1.7. Each plugin will be able to have his own templates
@login_required()
def index(request):
    return render_to_response(os.path.abspath(os.path.dirname(__file__) + '/templates/index.html'),
                              {
                                  'content': 'test'
                              },
                              context_instance=RequestContext(request))



from twisted.web import resource

class Root(resource.Resource):
    def __init__(self, wsgi_resource):
        resource.Resource.__init__(self)
        self.wsgi_resource = wsgi_resource

    def getChild(self, path, request):
        path0 = request.prepath.pop(0)
        request.postpath.insert(0, path0)
        return self.wsgi_resource


def verifyCallback(connection, x509, errnum, errdepth, ok):
    if not ok:
        print 'invalid cert from subject:', x509.get_subject()
        return False
    else:
        print "Certs are OK"
    return True




# -*- coding: UTF-8 -*-
import os
import json
from twisted.internet import reactor, ssl
from twisted.application import internet, service
from twisted.web import server, wsgi, static
from twisted.python import threadpool, log
from autobahn.twisted.websocket import WebSocketServerFactory
from autobahn.twisted.resource import WebSocketResource
from OpenSSL import SSL
import pkg_resources

if not os.path.exists('/etc/lisa/server/lisa.json'):
    configuration = json.load(open(pkg_resources.resource_filename(__name__, 'configuration/lisa.json.sample')))
else:
    configuration = json.load(open('/etc/lisa/server/lisa.json'))
path = os.path.abspath(__file__)
dir_path = os.path.dirname(path)

class ThreadPoolService(service.Service):
    def __init__(self, pool):
        self.pool = pool

    def startService(self):
        service.Service.startService(self)
        self.pool.start()

    def stopService(self):
        service.Service.stopService(self)
        self.pool.stop()


# Twisted Application Framework setup:
application = service.Application('LISA')

def makeService(config):
    from django.core.handlers.wsgi import WSGIHandler
    os.environ['DJANGO_SETTINGS_MODULE'] = 'lisa.server.web.weblisa.settings'

    global configuration

    if config['configuration']:
        configuration = json.load(open(config['configuration']))

    from lisa.server import libs

    # Creating MultiService
    multi = service.MultiService()
    pool = threadpool.ThreadPool()
    tps = ThreadPoolService(pool)
    tps.setServiceParent(multi)

    # Creating the web stuff
    resource_wsgi = wsgi.WSGIResource(reactor, tps.pool, WSGIHandler())
    root = libs.Root(resource_wsgi)
    staticrsrc = static.File('/'.join([dir_path,'web/interface/static']))
    root.putChild("static", staticrsrc)

    # Create the websocket
    if configuration['enable_secure_mode']:
        socketfactory = WebSocketServerFactory("wss://" + configuration['lisa_url'] + ":" +
                                               str(configuration['lisa_web_port_ssl']),debug=False)
    else:
        socketfactory = WebSocketServerFactory("ws://" + configuration['lisa_url'] + ":" +
                                               str(configuration['lisa_web_port']),debug=False)
    socketfactory.protocol = libs.WebSocketProtocol
    socketfactory.protocol.configuration, socketfactory.protocol.dir_path = configuration, dir_path
    socketresource = WebSocketResource(socketfactory)
    root.putChild("websocket", socketresource)

    # Configuring servers to launch
    if configuration['enable_secure_mode'] or configuration['enable_unsecure_mode']:
        if configuration['enable_secure_mode']:
            SSLContextFactoryEngine = ssl.DefaultOpenSSLContextFactory(
                os.path.normpath(dir_path + '/' + 'configuration/ssl/server.key'),
                os.path.normpath(dir_path + '/' + 'configuration/ssl/server.crt')
            )
            SSLContextFactoryWeb = ssl.DefaultOpenSSLContextFactory(
                os.path.normpath(dir_path + '/' + 'configuration/ssl/server.key'),
                os.path.normpath(dir_path + '/' + 'configuration/ssl/server.crt')
            )
            ctx = SSLContextFactoryEngine.getContext()
            ctx.set_verify(
                SSL.VERIFY_PEER | SSL.VERIFY_FAIL_IF_NO_PEER_CERT,
                libs.verifyCallback
            )
            # Since we have self-signed certs we have to explicitly
            # tell the server to trust them.
            with open(os.path.normpath(dir_path + '/' + 'configuration/ssl/server.pem'), 'w') as outfile:
                for file in os.listdir(os.path.normpath(dir_path + '/' + 'configuration/ssl/public/')):
                    with open(os.path.normpath(dir_path + '/' + 'configuration/ssl/public/'+file)) as infile:
                        for line in infile:
                            outfile.write(line)


            ctx.load_verify_locations(os.path.normpath(dir_path + '/' + 'configuration/ssl/server.pem'))
            internet.SSLServer(configuration['lisa_web_port_ssl'],
                               server.Site(root), SSLContextFactoryWeb).setServiceParent(multi)
            internet.SSLServer(configuration['lisa_engine_port_ssl'],
                               libs.LisaFactorySingleton.get(), SSLContextFactoryEngine).setServiceParent(multi)
        if configuration['enable_unsecure_mode']:
            # Serve it up:
            internet.TCPServer(configuration['lisa_web_port'], server.Site(root)).setServiceParent(multi)
            internet.TCPServer(configuration['lisa_engine_port'], libs.LisaFactorySingleton.get()).setServiceParent(multi)

    else:
        exit(1)
    libs.scheduler.setServiceParent(multi)
    multi.setServiceParent(application)
    libs.Initialize()
    return multi



from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponse
from models import Plugin, Rule, Cron
from django.contrib.auth.decorators import login_required
import requests, json
from lisa.server.web.weblisa.utils import method_restricted_to, is_ajax
from lisa.server.web.weblisa.settings import LISA_PATH

@method_restricted_to('GET')
@login_required()
def list(request):
    plugins = []
    metareq = requests.get('https://raw.github.com/Seraf/LISA-Plugins/master/plugin_list.json')
    if(metareq.ok):
        for item in json.loads(metareq.text or metareq.content):
            for plugin in Plugin.objects(name=item['name']):
                item['id'] = plugin.id
                item['enabled'] = plugin.enabled
                if plugin.version < item['version']:
                    item['upgrade'] = True
            #if os.path.exists(LISA_PATH + '/Plugins/' + item['name']):
            #    item['installed'] = True
            plugins.append(item)
    return render_to_response('list.html', {'Plugins': plugins},
                              context_instance=RequestContext(request))



# -*- coding: UTF-8 -*-
from lisa.server.plugins.IPlugin import IPlugin
import gettext
import inspect
import os

class TestPlugin(IPlugin):
    def __init__(self):
        super(TestPlugin, self).__init__()
        self.configuration_plugin = self.mongo.lisa.plugins.find_one({"name": "TestPlugin"})
        self.path = os.path.realpath(os.path.abspath(os.path.join(os.path.split(
            inspect.getfile(inspect.currentframe()))[0],os.path.normpath("../lang/"))))
        self._ = translation = gettext.translation(domain='testplugin',
                                                   localedir=self.path,
                                                   languages=[self.configuration_lisa['lang']]).ugettext

    def sayHello(self, jsonInput):
        return {"plugin": "TestPlugin",
                "method": "sayHello",
                "body": self._('Hello. How are you ?')
        }
